@@ -34,6 +34,68 @@
 #include "e-ews-item.h"
 #include "e-ews-item-change.h"
 
+#ifdef G_OS_WIN32
+
+static gchar *
+g_mkdtemp (gchar *tmpl,
+           gint mode)
+{
+	static const gchar letters[] = "ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789";
+	static const gint NLETTERS = sizeof (letters) - 1;
+	static gint counter = 0;
+	gchar *XXXXXX;
+	GTimeVal tv;
+	glong value;
+	gint count;
+
+	/* find the last occurrence of "XXXXXX" */
+	XXXXXX = g_strrstr (tmpl, "XXXXXX");
+
+	if (!XXXXXX || strncmp (XXXXXX, "XXXXXX", 6)) {
+		errno = EINVAL;
+		return NULL;
+	}
+
+	/* Get some more or less random data.  */
+	g_get_current_time (&tv);
+	value = (tv.tv_usec ^ tv.tv_sec) + counter++;
+
+	for (count = 0; count < 100; value += 7777, ++count) {
+		glong v = value;
+
+		/* Fill in the random bits.  */
+		XXXXXX[0] = letters[v % NLETTERS];
+		v /= NLETTERS;
+		XXXXXX[1] = letters[v % NLETTERS];
+		v /= NLETTERS;
+		XXXXXX[2] = letters[v % NLETTERS];
+		v /= NLETTERS;
+		XXXXXX[3] = letters[v % NLETTERS];
+		v /= NLETTERS;
+		XXXXXX[4] = letters[v % NLETTERS];
+		v /= NLETTERS;
+		XXXXXX[5] = letters[v % NLETTERS];
+
+		/* tmpl is in UTF-8 on Windows, thus use g_mkdir() */
+		if (g_mkdir (tmpl, mode) == 0)
+			return tmpl;
+
+		if (errno != EEXIST)
+			/* Any other error will apply also to other names we might
+			 *  try, and there are 2^32 or so of them, so give up now.
+			 */
+			return NULL;
+	}
+
+	/* We got out of the loop because we ran out of combinations to try.  */
+	errno = EEXIST;
+	return NULL;
+}
+
+#define mkdtemp(t) g_mkdtemp(t, 0700)
+
+#endif
+
 G_DEFINE_TYPE (EEwsItem, e_ews_item, G_TYPE_OBJECT)
 
 struct _EEwsContactFields {
@@ -148,6 +210,7 @@ struct _EEwsItemPrivate {
 	struct _EEwsTaskFields *task_fields;
 };
 
+static GObjectClass *parent_class = NULL;
 static void	ews_item_free_attendee (EwsAttendee *attendee);
 static void	ews_free_contact_fields (struct _EEwsContactFields *con_fields);
 
@@ -272,7 +335,26 @@ e_ews_item_dispose (GObject *object)
 	g_slist_free_full (priv->categories, g_free);
 	priv->categories = NULL;
 
-	G_OBJECT_CLASS (e_ews_item_parent_class)->dispose (object);
+	if (parent_class->dispose)
+		(* parent_class->dispose) (object);
+}
+
+static void
+e_ews_item_finalize (GObject *object)
+{
+	EEwsItem *item = (EEwsItem *) object;
+	EEwsItemPrivate *priv;
+
+	g_return_if_fail (E_IS_EWS_ITEM (item));
+
+	priv = item->priv;
+
+	/* clean up */
+	g_free (priv);
+	item->priv = NULL;
+
+	if (parent_class->finalize)
+		(* parent_class->finalize) (object);
 }
 
 static void
@@ -280,20 +362,25 @@ e_ews_item_class_init (EEwsItemClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
-	g_type_class_add_private (klass, sizeof (EEwsItemPrivate));
+	parent_class = g_type_class_peek_parent (klass);
 
 	object_class->dispose = e_ews_item_dispose;
+	object_class->finalize = e_ews_item_finalize;
 }
 
 static void
 e_ews_item_init (EEwsItem *item)
 {
-	item->priv = G_TYPE_INSTANCE_GET_PRIVATE (item, E_TYPE_EWS_ITEM, EEwsItemPrivate);
+	EEwsItemPrivate *priv;
 
-	item->priv->item_type = E_EWS_ITEM_TYPE_UNKNOWN;
+	/* allocate internal structure */
+	priv = g_new0 (EEwsItemPrivate, 1);
+	item->priv = priv;
 
-	item->priv->mapi_extended_tags = g_hash_table_new_full (g_direct_hash, g_direct_equal, NULL, g_free);
-	item->priv->mapi_extended_sets = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, (GDestroyNotify) g_hash_table_destroy);
+	priv->item_type = E_EWS_ITEM_TYPE_UNKNOWN;
+
+	priv->mapi_extended_tags = g_hash_table_new_full (g_direct_hash, g_direct_equal, NULL, g_free);
+	priv->mapi_extended_sets = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, (GDestroyNotify) g_hash_table_destroy);
 }
 
 static void
@@ -1765,7 +1852,7 @@ e_ews_item_dump_mime_content (EEwsItem *item,
 	tmpdir = g_path_get_dirname (tmpfilename);
 
 	dirname = g_build_filename (tmpdir, "XXXXXX", NULL);
-	if (!g_mkdtemp (dirname)) {
+	if (!mkdtemp (dirname)) {
 		g_warning ("Failed to create directory for attachment cache '%s': %s", dirname, g_strerror (errno));
 
 		g_free (tmpdir);
