@@ -25,7 +25,6 @@
 #include "e-ews-connection-utils.h"
 #include "e-ews-debug.h"
 #include "e-ews-notification.h"
-#include "e-soup-auth-negotiate.h"
 
 #define E_EWS_NOTIFICATION_GET_PRIVATE(obj)\
 	(G_TYPE_INSTANCE_GET_PRIVATE \
@@ -88,21 +87,8 @@ ews_notification_authenticate (SoupSession *session,
 
 	password = e_ews_connection_dup_password (connection);
 
-	if (password != NULL) {
+	if (password != NULL)
 		soup_auth_authenticate (auth, user, password);
-	} else {
-		/* The NTLM implementation in libsoup doesn't cope very well
-		 * with recovering from authentication failures (bug 703181).
-		 * So cancel the message now while it's in-flight, and we'll
-		 * get a shiny new connection for the next attempt. */
-		const char *scheme = soup_auth_get_scheme_name (auth);
-
-		if (!g_ascii_strcasecmp(scheme, "NTLM")) {
-			soup_session_cancel_message(notification->priv->soup_session,
-						    message,
-						    SOUP_STATUS_UNAUTHORIZED);
-		}
-	}
 
 	g_free (password);
 	g_free (user);
@@ -124,6 +110,14 @@ e_ews_notification_new (EEwsConnection *connection)
 
 	ews_settings = e_ews_connection_ref_settings (connection);
 
+	g_object_bind_property_full (
+		ews_settings, "auth-mechanism",
+		notification->priv->soup_session, "use-ntlm",
+		G_BINDING_SYNC_CREATE,
+		e_ews_connection_utils_auth_mech_to_use_ntlm,
+		NULL,
+		NULL, (GDestroyNotify) NULL);
+
 	g_object_unref (ews_settings);
 
 	return notification;
@@ -202,7 +196,6 @@ ews_notification_dispose (GObject *object)
 		g_signal_handlers_disconnect_by_func (
 			priv->soup_session,
 			ews_notification_authenticate, object);
-		g_clear_object (&priv->soup_session);
 	}
 
 	if (priv->cancellable != NULL)
@@ -220,38 +213,6 @@ ews_notification_dispose (GObject *object)
 	G_OBJECT_CLASS (e_ews_notification_parent_class)->dispose (object);
 }
 
-static GObject *
-ews_notification_constructor (GType gtype, guint n_properties,
-			      GObjectConstructParam *properties)
-{
-	GObject *obj = G_OBJECT_CLASS (e_ews_notification_parent_class)->
-		constructor (gtype, n_properties, properties);
-	EEwsNotificationPrivate *priv;
-	CamelEwsSettings *ews_settings;
-	EwsAuthType mech;
-
-	priv = E_EWS_NOTIFICATION_GET_PRIVATE (obj);
-	ews_settings = e_ews_connection_ref_settings (priv->connection);
-	mech = camel_ews_settings_get_auth_mechanism (ews_settings);
-
-	g_object_unref (ews_settings);
-
-	/* We used to disable Basic auth to avoid it getting in the way of
-	 * our GSSAPI hacks. But leave it enabled in the case where NTLM is
-	 * enabled, which is the default configuration. It's a useful fallback
-	 * which people may be relying on. */
-	if (mech == EWS_AUTH_TYPE_GSSAPI) {
-		soup_session_add_feature_by_type (priv->soup_session,
-						  E_SOUP_TYPE_AUTH_NEGOTIATE);
-		soup_session_remove_feature_by_type (priv->soup_session,
-						     SOUP_TYPE_AUTH_BASIC);
-	} else if (mech == EWS_AUTH_TYPE_NTLM)
-		soup_session_add_feature_by_type (priv->soup_session,
-						  SOUP_TYPE_AUTH_NTLM);
-
-	return obj;
-}
-
 static void
 e_ews_notification_class_init (EEwsNotificationClass *class)
 {
@@ -260,7 +221,6 @@ e_ews_notification_class_init (EEwsNotificationClass *class)
 	g_type_class_add_private (class, sizeof (EEwsNotificationPrivate));
 
 	object_class = G_OBJECT_CLASS (class);
-	object_class->constructor = ews_notification_constructor;
 	object_class->set_property = ews_notification_set_property;
 	object_class->get_property = ews_notification_get_property;
 	object_class->dispose = ews_notification_dispose;
@@ -285,10 +245,9 @@ e_ews_notification_init (EEwsNotification *notification)
 
 	notification->priv = E_EWS_NOTIFICATION_GET_PRIVATE (notification);
 
-	notification->priv->soup_session = soup_session_sync_new ();
-
-	soup_session_add_feature_by_type (notification->priv->soup_session,
-					  SOUP_TYPE_COOKIE_JAR);
+	notification->priv->soup_session = soup_session_sync_new_with_options (
+		SOUP_SESSION_USE_NTLM, TRUE,
+		NULL);
 
 	log_level = e_ews_debug_get_log_level ();
 	if (log_level >= 2) {
@@ -308,21 +267,19 @@ e_ews_notification_subscribe_folder_sync (EEwsNotification *notification,
 					  gchar **subscription_id,
 					  GCancellable *cancellable)
 {
+	CamelEwsSettings *ews_settings;
 	ESoapMessage *msg;
 	ESoapResponse *response;
 	ESoapParameter *param, *subparam;
 	GError *error = NULL;
 	GSList *l;
+	gchar *auth_mech = NULL;
 	guint event_type;
 	xmlDoc *doc;
 
 	g_return_val_if_fail (notification != NULL, FALSE);
 	g_return_val_if_fail (notification->priv != NULL, FALSE);
 
-	/* Can happen during process shutdown */
-	if (!notification->priv->connection)
-		return FALSE;
-
 	msg = e_ews_message_new_with_header (
 		e_ews_connection_get_uri (notification->priv->connection),
 		e_ews_connection_get_impersonate_user (notification->priv->connection),
@@ -368,13 +325,25 @@ e_ews_notification_subscribe_folder_sync (EEwsNotification *notification,
 
 	soup_message_body_set_accumulate (SOUP_MESSAGE (msg)->response_body, TRUE);
 
+	ews_settings = e_ews_connection_ref_settings (notification->priv->connection);
+
+	g_object_get (G_OBJECT (ews_settings), "auth-mechanism", &auth_mech, NULL);
+	if (g_strcmp0 (auth_mech, "GSSAPI") == 0)
+		e_ews_connection_utils_setup_msg_gssapi_auth (
+			notification->priv->connection,
+			notification->priv->soup_session,
+			SOUP_MESSAGE (msg));
+
+	g_object_unref (ews_settings);
+	g_free (auth_mech);
+
 	if (g_cancellable_is_cancelled (cancellable)) {
 		g_object_unref (msg);
 		return FALSE;
 	}
 
-	soup_session_send_message (notification->priv->soup_session, SOUP_MESSAGE (msg));
-	if (!SOUP_STATUS_IS_SUCCESSFUL (SOUP_MESSAGE (msg)->status_code)) {
+	if (!SOUP_STATUS_IS_SUCCESSFUL (
+		soup_session_send_message (notification->priv->soup_session, SOUP_MESSAGE (msg)))) {
 		g_object_unref (msg);
 		return FALSE;
 	}
@@ -440,10 +409,6 @@ e_ews_notification_unsubscribe_folder_sync (EEwsNotification *notification,
 	g_return_val_if_fail (notification != NULL, FALSE);
 	g_return_val_if_fail (notification->priv != NULL, FALSE);
 
-	/* Can happen during process shutdown */
-	if (!notification->priv->connection)
-		return FALSE;
-
 	msg = e_ews_message_new_with_header (
 		e_ews_connection_get_uri (notification->priv->connection),
 		e_ews_connection_get_impersonate_user (notification->priv->connection),
@@ -462,8 +427,8 @@ e_ews_notification_unsubscribe_folder_sync (EEwsNotification *notification,
 
 	soup_message_body_set_accumulate (SOUP_MESSAGE (msg)->response_body, TRUE);
 
-	soup_session_send_message (notification->priv->soup_session, SOUP_MESSAGE (msg));
-	if (!SOUP_STATUS_IS_SUCCESSFUL (SOUP_MESSAGE (msg)->status_code)) {
+	if (!SOUP_STATUS_IS_SUCCESSFUL (
+		soup_session_send_message (notification->priv->soup_session, SOUP_MESSAGE (msg)))) {
 		g_object_unref (msg);
 		return FALSE;
 	}
@@ -625,8 +590,7 @@ ews_notification_fire_events_from_response (EEwsNotification *notification,
 	}
 
 	if (events != NULL) {
-		if (notification->priv->connection)
-			g_signal_emit_by_name (notification->priv->connection, "server-notification", events);
+		g_signal_emit_by_name (notification->priv->connection, "server-notification", events);
 		g_slist_free_full (events, (GDestroyNotify) e_ews_notification_event_free);
 	}
 }
@@ -743,8 +707,8 @@ e_ews_notification_get_events_sync (EEwsNotification *notification,
 		SOUP_MESSAGE (msg), "got-chunk",
 		G_CALLBACK (ews_notification_soup_got_chunk), notification);
 
-	soup_session_send_message (notification->priv->soup_session, SOUP_MESSAGE (msg));
-	ret = SOUP_STATUS_IS_SUCCESSFUL (SOUP_MESSAGE (msg)->status_code);
+	ret = SOUP_STATUS_IS_SUCCESSFUL (
+		soup_session_send_message (notification->priv->soup_session, SOUP_MESSAGE (msg)));
 
 	g_signal_handler_disconnect (msg, handler_id);
 	g_object_unref (msg);
